==> alita_sdk-0.1.64-py3-none-any.whl: alita_sdk/agents/autogen_agent.py <==
from typing import Sequence, Union, Any, Optional
from traceback import format_exc
from json import dumps
from langchain_core.agents import AgentAction, AgentFinish
from langchain_core.prompts import BasePromptTemplate
from langchain_core.messages import BaseMessage, AIMessage, HumanMessage
from langchain_core.tools import BaseTool
from langchain_core.utils.function_calling import convert_to_openai_tool
from langchain.agents.openai_assistant.base import OutputType
from langchain_core.runnables import RunnableSerializable, ensure_config
from langchain.tools.render import ToolsRenderer
from langchain_core.load import dumpd
from .mixedAgentRenderes import render_react_text_description_and_args
from .mixedAgentRenderes import conversation_to_messages, format_to_langmessages, convert_message_to_json
from langchain_core.callbacks import CallbackManager
from langchain_core.runnables import RunnableConfig, RunnableSerializable, ensure_config
from uuid import uuid4
from langchain_core.outputs import LLMResult, ChatGenerationChunk
from langchain_core.outputs.run_info import RunInfo
from langchain_core.outputs.generation import Generation
from ..clients.constants import ALITA_OUTPUT_FORMAT

from autogen import ConversableAgent

class AutoGenAssistantRunnable(RunnableSerializable):
    client: Optional[Any]
    assistant: Optional[Any]
    chat_history: list[BaseMessage] = []
    agent_type:str = "autogen"

    @classmethod
    def create_assistant(
        cls,
        client: Any,
        prompt: BasePromptTemplate,
        tools: Sequence[Union[BaseTool, dict]],
        tools_renderer: Optional[ToolsRenderer] = render_react_text_description_and_args,
    ) -> RunnableSerializable:
        tools_dict = []
        functional_calls = {}
        for tool in tools:
            functional_calls[tool.name] = tool
            
        client['tools'] = [convert_to_openai_tool(tool) for tool in tools]
        assistant = ConversableAgent(
            'chatbot',
            llm_config=client,
            function_map=functional_calls,
            system_message=prompt.messages[0].content,
            chat_messages=convert_message_to_json(prompt.messages[1:])
        )
        return cls(client=client, assistant=assistant, chat_history=[])
        
    
    def invoke(self, input: dict, config: RunnableConfig | None = None) -> OutputType:
        run_id = uuid4()
        config = ensure_config(config)
        callback_manager = CallbackManager.configure(
            inheritable_callbacks=config.get("callbacks"),
            inheritable_tags=config.get("tags"),
            inheritable_metadata=config.get("metadata"),
        )
        run_manager = callback_manager.on_chain_start(
            dumpd(self), input, name=config.get("run_name"), run_id=run_id
        )
        messages = []
        if input.get("intermediate_steps"):
            messages = format_to_langmessages(input["intermediate_steps"])
        
        try:
            user_messages = [] 
            if input.get('input', input.get('content')):
                user_messages.append(HumanMessage(content=input.get('input', input.get('content'))))
            msgs = self.chat_history + \
                conversation_to_messages(input["chat_history"]) + \
                     user_messages + \
                        messages
            llm_manager = callback_manager.on_llm_start(dumpd(self), [msgs[-1].content], run_id=run_id)
            run = self._create_thread_and_run(msgs)
            response = self._get_response(run)
            try:
                log = response.log
            except AttributeError:
                log = response
            llm_manager[0].on_llm_new_token(
                token=str(log), chunk=ChatGenerationChunk(
                    text=str(log), message=AIMessage(content=str(log))
                ))
            llm_manager[0].on_llm_end(LLMResult(generations=[[Generation(text=str(log))]], 
                                                run=[RunInfo(run_id=run_id)]))
        except BaseException as e:
            run_manager.on_chain_error(e, metadata=format_exc())
            raise e
        else:
            run_manager.on_chain_end(response)
            return response
        
    
    def _create_thread_and_run(self, messages: list[BaseMessage]) -> Any:
        messages = convert_message_to_json(messages)
        return self.assistant.generate_reply(messages=messages)
    
    def _get_response(self, run: Union[str, dict]) -> Any:
        if isinstance(run, dict):
            return AgentAction(run['tool_calls'][0]['function']['name'], 
                               run['tool_calls'][0]['function']['arguments'], 
                               dumps(run))
        else:
            return AgentFinish({"output": run}, log=run)

==> alita_sdk-0.1.80-py3-none-any.whl: alita_sdk/agents/autogen_agent.py <==
from typing import Sequence, Union, Any, Optional
from json import dumps, loads
from langchain_core.agents import AgentAction, AgentFinish
from langchain_core.prompts import BasePromptTemplate
from langchain_core.messages import BaseMessage
from langchain_core.tools import BaseTool
from langchain_core.utils.function_calling import convert_to_openai_tool
from langchain.agents.output_parsers.openai_tools import ToolAgentAction
from langchain_core.runnables import RunnableSerializable
from langchain_core.tools.render import ToolsRenderer
from .mixedAgentRenderes import render_react_text_description_and_args
from .mixedAgentRenderes import convert_message_to_json
from .alita_agent import AlitaAssistantRunnable
        
from autogen import ConversableAgent

class AutoGenAssistantRunnable(AlitaAssistantRunnable):
    client: Optional[Any]
    assistant: Optional[Any]
    chat_history: list[BaseMessage] = []
    agent_type:str = "autogen"

    @classmethod
    def create_assistant(
        cls,
        client: Any,
        prompt: BasePromptTemplate,
        tools: Sequence[Union[BaseTool, dict]],
        tools_renderer: Optional[ToolsRenderer] = render_react_text_description_and_args,
    ) -> RunnableSerializable:
        functional_calls = {}
        for tool in tools:
            functional_calls[tool.name] = tool

        client['tools'] = [convert_to_openai_tool(tool) for tool in tools]
        assistant = ConversableAgent(
            'chatbot',
            llm_config=client,
            function_map=functional_calls,
            system_message=prompt.messages[0].content,
            chat_messages=convert_message_to_json(prompt.messages[1:])
        )
        return cls(client=client, assistant=assistant, agent_type='autogen', chat_history=[])
    
    def _create_thread_and_run(self, messages: list[BaseMessage]) -> Any:
        messages = convert_message_to_json(messages)
        return self.assistant.generate_reply(messages=messages)
    
    def _get_response(self, run: Union[str, dict]) -> Any:
        if isinstance(run, dict):
            tool_calls = run.get('tool_calls', [])
            if not tool_calls:
                return AgentFinish({"output": run['content']}, log=run)
            actions = []
            for tool_call in tool_calls:
                actions.append(AgentAction(
                    tool=tool_call['function']['name'], 
                    tool_input=loads(tool_call['function'].get('arguments', '{}')), 
                    log=dumps(tool_call),
                    tool_call_id=tool_call['id']))
            return actions
            
        else:
            return AgentFinish({"output": run}, log=run)
